# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition constants
  -------------------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''


import os
from schema import Schema, Optional, Or


DEFAULT_CONFIG = {'algorithm': {}, 'manager': {}}

DEFAULT_LOGPATH = '/tmp'

DEFAULT_HOME = '/'.join([os.environ.get('HOME', '/'), '.intuition'])

# https://github.com/halst/schema
# NOTE Even better ? https://github.com/j2labs/schematics
# TODO More strict validation
CONFIG_SCHEMA = Schema({
    'universe': basestring,
    'index': object,
    # Optional('id'): Use(basestring, error='invalid identity'),
    Optional('id'): basestring,
    #Optional('live'): bool,
    Optional('_id'): object,
    Optional('__v'): object,
    'modules': {
        'algorithm': basestring,
        # TODO It will be at least one
        Optional('backtest'): basestring,
        Optional('live'): basestring,
        Optional('manager'): Or(basestring, None)
    }
})


# Some urls used by the data module to fetch securities infos
FINANCE_URLS = {
    'yahoo_hist': 'http://ichart.yahoo.com/table.csv',
    'yahoo_infos': 'http://finance.yahoo/q/pr',
    'google_prices': 'http://www.google.com/finance/getprices',
    'snapshot_google_light': 'http://www.google.com/finance/info',
    'info_lookup': ('http://d.yimg.com/autoc.finance.yahoo.com/autoc?' +
                    'query={}&callback=YAHOO.Finance.SymbolSuggest.ssCallback')
}



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition Context api
  ---------------------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

import abc
import datetime as dt
import dna.logging
import intuition.utils


def parse_storage(storage):
    tmp_uri = storage.split('?')
    path = tmp_uri[0].split('/')[1:]
    uri = tmp_uri[0].split('/')[0]

    params = {}
    if len(tmp_uri) > 1:
        tmp_params = tmp_uri[1].split('&')
        for item in tmp_params:
            if item.find('=') > 0:
                k, v = item.split('=')
                params[k] = v
            else:
                params[item] = True

    return {
        'uri': uri,
        'path': path,
        'params': params
    }


class ContextFactory():
    '''
    Context loaders give to Intuition everything it needs to know about user
    configuration. It also provides some methods to make the setup process
    easier.
    '''

    __metaclass__ = abc.ABCMeta

    def __init__(self, storage):
        self.log = dna.logging.logger(__name__)
        self.initialize(parse_storage(storage))

    def initialize(self, storage):
        ''' Users should overwrite this method '''
        pass

    @abc.abstractmethod
    def load(self, uri, path, params):
        ''' Users should overwrite this method '''
        pass

    def _normalize_dates(self, context):
        '''
        Build a timeline from given (or not) start and end dates
        '''
        if 'start' in context:
            if isinstance(context['start'], dt.date):
                context['start'] = dt.date.strftime(
                    context['start'], format='%Y-%m-%d')
        if 'end' in context:
            if isinstance(context['end'], dt.date):
                context['end'] = dt.date.strftime(
                    context['end'], format='%Y-%m-%d')

        trading_dates = intuition.utils.build_trading_timeline(
            context.pop('start', None), context.pop('end', None))

        context['index'] = trading_dates

    def _normalize_data_types(self, strategy):
        ''' some contexts only retrieves strings, giving back right type '''
        for k, v in strategy.iteritems():
            if not isinstance(v, str):
                # There is probably nothing to do
                continue
            if v == 'true':
                strategy[k] = True
            elif v == 'false' or v is None:
                strategy[k] = False
            else:
                try:
                    if v.find('.') > 0:
                        strategy[k] = float(v)
                    else:
                        strategy[k] = int(v)
                except ValueError:
                    pass

    def build(self):
        context = self.load()

        algorithm = context.pop('algorithm', {})
        manager = context.pop('manager', {})
        data = context.pop('data', {})

        if context:
            self._normalize_dates(context)

        self._normalize_data_types(algorithm)
        self._normalize_data_types(manager)
        self._normalize_data_types(data)

        strategy = {
            'algorithm': algorithm,
            'manager': manager,
            'data': data
        }

        return context, strategy



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition test utilities
  ------------------------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

import pandas as pd
import pytz
from random import random
from intuition.data.universe import Market
from intuition.api.context import ContextFactory
from intuition.api.datafeed import HybridDataFactory
from intuition.api.portfolio import PortfolioFactory
from intuition.api.algorithm import TradingFactory


class FakeContext(ContextFactory):
    ''' Returns fake configuration '''

    def initialize(self, storage):
        self.storage = storage
        self.raise_error = not bool(storage['params'].get('valid', True))
        self.bad_format = bool(storage['params'].get('format', False))

    def load(self):
        if self.raise_error:
            raise ValueError
        return {
            'universe': 'forex,5',
            'index': pd.date_range('2012/01/01', '2012/02/01'),
            'modules': {
                'algorithm': 'fake_algorithm',
                'manager': 'fake_manager',
                'backtest': 'fake_backtest',
                'live': 'fake_live'
            }
        } if not self.bad_format else {}


class TestAlgorithm(TradingFactory):
    ''' Returns minimal algorithm for tests '''
    warmed_data = None

    def initialize(self, properties):
        self.properties = properties

    def warm(self, data):
        self.warmed_data = data

    def event(self, data):
        return {}


class TestPortfolio(PortfolioFactory):
    ''' Returns minimal portfolio for tests '''
    initialized = False

    def initialize(self, properties):
        self.initialized = True

    def optimize(self, to_buy, to_sell):
        if 'raise_fake_error' in self.properties:
            raise ValueError('whatever error')
        return {'date': self.date, 'buy': to_buy}, 0, 1


class FakeBacktestDatasource(object):

    def __init__(self, sids, properties):
        pass

    @property
    def mapping(self):
        return {
            'backtest': (lambda x: True, 'sid'),
            'dt': (lambda x: x, 'dt'),
            'sid': (lambda x: x, 'sid'),
            'price': (float, 'price'),
            'volume': (int, 'volume'),
        }

    def get_data(self, sids, start, end):
        index = pd.date_range(start, end, tz=pytz.utc)
        return pd.DataFrame({sid: [random()] * len(index)
                            for sid in sids}, index=index)


class FakePanelBacktestDatasource(object):

    def __init__(self, sids, properties):
        pass

    @property
    def mapping(self):
        return {
            'backtest': (lambda x: True, 'sid'),
            'dt': (lambda x: x, 'dt'),
            'sid': (lambda x: x, 'sid'),
            'price': (float, 'price'),
            'low': (float, 'low'),
            'high': (float, 'high'),
            'volume': (int, 'volume'),
        }

    def get_data(self, sids, start, end):
        index = pd.date_range(start, end, tz=pytz.utc)
        fake_data = {}
        for sid in sids:
            fake_data[sid] = pd.DataFrame(
                {field: [random()] * len(index)
                 for field in ['price', 'low', 'high', 'volume']}, index=index)
        return pd.Panel(fake_data)


class FakePanelWithoutVolumeBacktestDatasource(object):

    def __init__(self, sids, properties):
        pass

    def get_data(self, sids, start, end):
        index = pd.date_range(start, end, tz=pytz.utc)
        fake_data = {}
        for sid in sids:
            fake_data[sid] = pd.DataFrame(
                {field: [random()] * len(index)
                 for field in ['price', 'low', 'high']}, index=index)
        return pd.Panel(fake_data)


class FakeLiveDatasource(object):

    def __init__(self, sids, properties):
        pass

    @property
    def mapping(self):
        return {
            'live': True
        }

    def get_data(self, sids, start, end):
        return pd.DataFrame()


def build_fake_hybridDataFactory(start='2012/01/01',
                                 end='2012/01/07',
                                 universe='forex,3'):
    test_index = pd.date_range(start, end, tz=pytz.utc)
    market_ = Market()
    market_.parse_universe_description(universe)
    return HybridDataFactory(
        universe=market_,
        index=test_index,
        backtest=FakeBacktestDatasource
    )



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  data informations
  -----------------

  :copyright (c) 2014 Xavier Bruhiere.
  :license: Apache2.0, see LICENSE for more details.
'''


"""
# World exchanges caracteristics
Exchanges = {
    # Market code, from yahoo stock code to google market code (needed for
    # reliable download)
    # Later Londres = LON
    #FIXME Forex is temporary
    'cac40': {'symbol': '^FCHI',
              'timezone': 'Europe/London',
              'code': 1001,
              'google_market': 'EPA'},
    'forex': {'symbol': '^GSPC',
              'timezone': 'US/Eastern',
              'code': 1002,
              'indexes': []},
    'nasdaq': {'symbol': '^GSPC',
               'timezone': 'US/Eastern',
               'code': 1003,
               'indexes': ['nasdaq', 'nyse'],
               'google_market': 'NASDAQ'},
    'nyse': {'symbol': '^GSPC',
             'timezone': 'US/Eastern',
             'code': 1004,
             'google_market': 'NYSE'}
}
"""


yahooCode = {'ask': 'a', 'average daily volume': 'a2', 'ask size': 'a5',
             'bid': 'b', 'ask rt': 'b2', 'bid rt': 'b3', 'dividend yield': 'y',
             'book value': 'b4', 'bid size': 'b6', 'change and percent': 'c',
             'change': 'c1', 'commission': 'c3', 'change rt': 'c6',
             'after hours change rt': 'c8', 'dividend': 'd',
             'last trade date': 'd1', 'trade date': 'd2', 'earnings': 'e',
             'error': 'e1', 'eps estimate year': 'e7',
             'eps estimate next year': 'e8', 'eps estimate next quarter': 'e9',
             'float shares': 'f6', 'day low': 'g', 'day high': 'h',
             '52-week low': 'j', '52-week high': 'k',
             'holdings gain percent': 'g1', 'annualized gain': 'g3',
             'holdings gain': 'g4', 'holdings gain percent rt': 'g5',
             'holdings gain rt': 'g6', 'more info': 'i', 'order book rt': 'i5',
             'market capitalization': 'j1', 'market cap rt': 'j3',
             'EBITDA': 'j4', 'change from 52-week': 'j5',
             'percent change from 52-week low': 'j6',
             'last trade rt with time': 'k1', 'change percent rt': 'k2',
             'last trade size': 'k3', 'change from 52-week high': 'k4',
             'percent change from 52-week high': 'k5',
             'last trade with time': 'l', 'last trade price': 'l1',
             'high limit': 'l2', 'low limit': 'l3', 'day range': 'm',
             'day range rt': 'm2', '50-day ma': 'm3', '200-day ma': 'm4',
             'percent change from 50-day ma': 'm8', 'name': 'n', 'notes': 'n4',
             'open': 'o', 'previous close': 'p', 'price paid': 'p1',
             'change percent': 'p2', 'price/sales': 'p5', 'price/book': 'p6',
             'ex-dividend date': 'q', 'p/e ratio': 'r', 'dividend date': 'r1',
             'p/e ratio rt': 'r2', 'peg ratio': 'r5',
             'price/eps estimate year': 'r6',
             'price/eps estimate next year': 'r7', 'symbol': 's',
             'shares owned': 's1', 'short ratio': 's7',
             'last trade time': 't1', 'trade links': 't6',
             'ticker trend': 't7', '1 year target price': 't8', 'volume': 'v',
             'holdings value': 'v1', 'holdings value rt': 'v7',
             '52-week range': 'w', 'day value change': 'w1',
             'day value change rt': 'w4', 'stock exchange': 'x'}



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Universe manager
  ----------------

  It knows everything about market and smartly handle user input

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''


import os
import random
import pytz
import yaml
import datetime as dt
import pandas as pd
import dateutil.parser
import dna.logging
from intuition.errors import LoadMarketSchemeFailed

log = dna.logging.logger(__name__)


# TODO Move it to utils
def timezone_aware_parser(date, timezone='UTC'):
    return pytz.timezone(timezone)\
        .localize(dateutil.parser.parse(date))\
        .astimezone(pytz.utc)


class Tick(object):
    '''
    Represent a convenient clock, utc aware, through 3 attributes:
      - start : datetime low limit [default is midnight]
      - delta : timestamp between ticks [default is 1h]
      - end   : datetime high limit [default is next midnight]
    '''

    def __init__(self, **kwargs):
        _default_start = pd.datetools.normalize_date(
            dt.datetime.now(tz=pytz.utc))
        _default_end = pd.datetools.normalize_date(
            dt.datetime.now(tz=pytz.utc) + dt.timedelta(days=1))
        _default_delta = dt.timedelta(hours=1)

        self.start = kwargs.get('start', _default_start)
        self.end = kwargs.get('end', _default_end)
        self.delta = kwargs.get('delta', _default_delta)
        self._tz = kwargs.get('tz', 'UTC')

    @property
    def tack(self):
        current_date = self.start
        while current_date < self.end:
            yield current_date
            current_date += self.delta

    def _parse_punctual(self, human_time):
        '''
        Translate human-like approximations to datetime
          - "at opening" => self.start + 5 minutes
          - "at midday" => self.end + self.start / 2
          - "at opening" => self.start + 5 minutes
          - "at X o'clock" => parse(Xh00)
          - "at xxhxx" => parse(xxhxx)
        '''
        tokens = human_time.split()
        self.delta = dt.timedelta(days=1)

        if human_time.find('opening') > 0:
            self.start = self.start + dt.timedelta(minutes=5)

        elif human_time.find('midday') > 0:
            self.start += (self.end - self.start) / 2

        elif human_time.find('closing') > 0:
            self.start = self.end - dt.timedelta(minutes=5)

        elif human_time.find('clock') > 0:
            today_start = timezone_aware_parser(tokens[1] + 'h00', self._tz)
            self.start = self.start.replace(
                hour=today_start.hour, minute=today_start.minute)

        elif human_time.find('h') > 0:
            today_start = timezone_aware_parser(tokens[1], self._tz)
            self.start = self.start.replace(
                hour=today_start.hour, minute=today_start.minute)

        else:
            raise ValueError('unable to parse ' + human_time)

    def _parse_constant_frequency(self, human_time):
        ''' Parse an input like "every 30 {hours,minutes}" '''
        tokens = human_time.split()

        if tokens[-1].lower() == 'minutes':
            self.delta = dt.timedelta(minutes=float(tokens[1]))

        elif tokens[-1].lower() == 'hours':
            self.delta = dt.timedelta(hours=float(tokens[1]))

        else:
            raise ValueError('unable to parse ' + human_time)

    def parse(self, human_time):
        ''' Set self.start and self.delta from a human timestamp input '''
        if human_time.find('every') >= 0:
            self._parse_constant_frequency(human_time)

        elif human_time.find('at') >= 0:
            self._parse_punctual(human_time)

        else:
            raise ValueError('unable to parse ' + human_time)


class Market(object):
    '''
    Knows everything about market and smartly handle user input
    '''

    # TODO Read default from market.yml or set by the user
    benchmark = '^GSPC'
    default_timezone = 'US/Eastern'
    default_open = '00h01'
    default_close = '23h59'
    exchange = None
    raw_description = None
    scheme_path = os.path.expanduser('~/.intuition/data/market.yml')

    def __init__(self):
        log.info('loading market scheme')
        self._load_market_scheme()

    def _load_market_scheme(self):
        ''' Load market yaml description '''
        try:
            self.scheme = yaml.load(open(self.scheme_path, 'r'))
        except Exception, error:
            raise LoadMarketSchemeFailed(reason=error)

    def _extract_forex(self):
        self.scheme = self.scheme['currencies']['forex']
        return self.scheme['pairs']

    def _extract_cac40(self, market):
        market_scheme = self.scheme
        # Walk the market structure
        for key in market[:-1]:
            market_scheme = market_scheme[key]

        self.scheme = market_scheme
        self.benchmark = market_scheme['benchmark']
        return map(
            lambda x: x + '.pa',
            market_scheme[market[-1]].keys())

    @property
    def timezone(self):
        return self.scheme.get('timezone') or self.default_timezone

    @property
    def open(self):
        safe_open = self.scheme.get('open') or self.default_open
        return timezone_aware_parser(safe_open, self.timezone)

    @property
    def close(self):
        # Check if a specific market scheme was set
        safe_close = self.scheme.get('close') or self.default_close
        return timezone_aware_parser(safe_close, self.timezone)

    def _lookup_sids(self, market, limit=-1):
        market = market.split(':')

        try:
            if 'forex' in market:
                sids = self._extract_forex()
            elif 'paris' in market:
                sids = self._extract_cac40(market)
            else:
                raise NotImplementedError(
                    'market {} not supported'.format(market))
        except Exception, error:
            raise LoadMarketSchemeFailed(reason=error)

        random.shuffle(sids)
        return sids[:limit] if limit > 0 else sids

    # TODO multi exchange, sector and other criterias
    def parse_universe_description(self, description):
        ''' Semantic

           - 'sid1,sid2,sid2,...'
           - 'exchange' : every sids of the exchange
           - 'exchange,n' : n random sids of the exchange

        where exchange is a combination of 'type:index:submarket'
        '''
        self.raw_description = description
        description = description.split(',')
        self.exchange = description[0]

        n = int(description[1]) if len(description) == 2 else -1
        self.sids = self._lookup_sids(description[0], n)

    def _detect_exchange(self, description):
        ''' Guess from the description and the market scheme '''
        pass

    def __str__(self):
        return '<sids: {} exchange: {} timezone: {} benchmark: {}'.format(
            self.sids, self.exchange, self.timezone, self.benchmark)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition app entry point
  --------------------------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

import sys
from intuition.cli import main


if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition configuration
  -----------------------

  Configure intuition at launch time

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

import os
import argparse
import pytz
from schematics.types import StringType, URLType
import dna.logging
import dna.utils
from intuition import __version__, __licence__
import intuition.constants
import intuition.utils as utils
import intuition.data.universe as universe
from intuition.errors import InvalidConfiguration

log = dna.logging.logger(__name__)


def parse_commandline():
    parser = argparse.ArgumentParser(
        description='Intuition, the terrific trading system')
    parser.add_argument('-V', '--version',
                        action='version',
                        version='%(prog)s v{} Licence {}'.format(
                            __version__, __licence__),
                        help='Print program version')
    parser.add_argument('-v', '--showlog',
                        action='store_true',
                        help='Print logs on stdout')
    parser.add_argument('-b', '--bot',
                        action='store_true',
                        help='Allows the algorithm to process orders')
    parser.add_argument('-c', '--context',
                        action='store', default='file::conf.yaml',
                        help='Provides the way to build context')
    parser.add_argument('-i', '--id',
                        action='store', default='gekko',
                        help='Customize the session id')
    args = parser.parse_args()

    # Dict will be more generic to process than args namespace
    return {
        'session': args.id,
        'context': args.context,
        'showlog': args.showlog,
        'bot': args.bot
    }


class Context(object):
    ''' Load and control configuration '''

    def __init__(self, access):
        # Hold infos to reach the config formatted like an url path
        # <path.to.module>://<ip>:<port>/<access/...?<key>=<value>...
        StringType(regex='.*://\w').validate(access)
        self._ctx_module = access.split('://')[0]
        self._ctx_infos = access.split('://')[1]
        URLType().validate('http://{}'.format(self._ctx_infos))

    def __enter__(self):
        # Use the given context module to grab configuration
        Loader = utils.intuition_module(self._ctx_module)
        log.info('building context',
                 driver=self._ctx_module, data=self._ctx_infos)
        config, strategy = Loader(self._ctx_infos).build()

        # TODO Validate strategy as well
        self._validate(config)

        # From a human input (forex,4), setup a complete market structure
        market_ = universe.Market()
        market_.parse_universe_description(config.pop('universe'))

        return {'config': config, 'strategy': strategy, 'market': market_}

    def __exit__(self, type, value, traceback):
        pass

    def _validate(self, config):
        log.info('validating configuration', config=config)
        try:
            # Check if needed informations are here
            assert intuition.constants.CONFIG_SCHEMA.validate(config)
            # Index Check
            assert config['index'].size, 'No trading timeline'
            assert (config['index'].tzinfo == pytz.utc), 'Invalid timezone'
        except Exception as error:
            raise InvalidConfiguration(config=config, reason=error)


def logfile(session_id):
    # Create a special file for the session, with a fallback in /tmp
    log_path = os.path.expanduser('~/.intuition/logs')
    log_path = log_path if os.path.exists(log_path) \
        else intuition.constants.DEFAULT_LOGPATH
    return '{}/{}.log'.format(log_path, session_id)



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Data related utilities
  ----------------------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

import pandas as pd


# TODO Make it a decorator
def clean_sid(sid):
    sid = str(sid).lower()
    # Remove market extension
    dot_pos = sid.find('.')
    sid = sid[:dot_pos] if dot_pos > 0 else sid
    # Remove forex slash
    return sid.replace('/', '')


def apply_mapping(raw_row, mapping):
    '''
    Override this to hand craft conversion of row.
    '''
    row = {target: mapping_func(raw_row[source_key])
           for target, (mapping_func, source_key)
           in mapping.fget().items()}
    return row


def invert_dataframe_axis(fct):
    '''
    Make dataframe index column names,
    and vice et versa
    '''
    def inner(*args, **kwargs):
        df_to_invert = fct(*args, **kwargs)
        return pd.DataFrame(df_to_invert.to_dict().values(),
                            index=df_to_invert.to_dict().keys())
    return inner


# NOTE with_market symbol, build jxr.pa => JXR:EPA
def use_google_symbol(fct):
    '''
    Removes ".PA" or other market indicator from yahoo symbol
    convention to suit google convention
    '''
    def decorator(symbols):
        google_symbols = []

        # If one symbol string
        if isinstance(symbols, str):
            symbols = [symbols]

        symbols = sorted(symbols)
        for symbol in symbols:
            dot_pos = symbol.find('.')
            google_symbols.append(
                symbol[:dot_pos] if (dot_pos > 0) else symbol)

        data = fct(google_symbols)
        # NOTE Not very elegant
        data.columns = [s for s in symbols if s.split('.')[0] in data.columns]
        return data
    return decorator


def fractionate_request(fct):
    def inner(symbols, **kwargs):
        tolerance_window = 10
        cursor = 0
        data = {}
        df = pd.DataFrame()
        while cursor < len(symbols):
            if cursor + tolerance_window > len(symbols):
                limit = len(symbols)
            else:
                limit = cursor + tolerance_window
            symbols_fraction = symbols[cursor:limit]
            cursor += tolerance_window
            data_fraction = fct(symbols_fraction, **kwargs)
            for sid, df in data_fraction.iteritems():
                if not df.empty:
                    data[sid] = df

        if isinstance(df, pd.Series):
            data = pd.DataFrame(data)
        elif isinstance(df, pd.DataFrame):
            data = pd.Panel(data)
        else:
            raise NotImplementedError(type(df))
        return data.fillna(method='pad')
    return inner



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition financial library
  ---------------------------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''


from pandas.core.datetools import BDay
import numpy as np
import math
import datetime as dt


class DailyReturn(object):
    '''
    >>> returns = DailyReturn()
    >>> returns(1.34)
    0.0
    >>> returns(1.37)
    2.2388059701492558
    '''
    _last_price = None

    def __call__(self, new_price):
        if self._last_price:
            daily_return = ((new_price - self._last_price)
                            / self._last_price) * 100
        else:
            daily_return = 0.0

        self._last_price = new_price

        return daily_return


# NOTE This is temporary copied from QSTK library
#     which will be more used in the future
def qstk_get_sharpe_ratio(rets, risk_free=0.00):
    """
    @summary Returns the daily Sharpe ratio of the returns.
    @param rets: 1d numpy array or fund list of daily returns (centered on 0)
    @param risk_free: risk free returns, default is 0%
    @return Annualized rate of return, not converted to percent
    """
    f_dev = np.std(rets, axis=0)
    f_mean = np.mean(rets, axis=0)

    f_sharpe = (f_mean * 252 - risk_free) / (f_dev * np.sqrt(252))

    return f_sharpe


def moving_average(data, periods, method='simple'):
    """
    compute a <periods> period moving average.
    method is 'simple' | 'exponential'
    """
    data = np.asarray(data)
    if method == 'simple':
        weights = np.ones(periods)
    else:
        weights = np.exp(np.linspace(-1., 0., periods))

    weights /= weights.sum()

    mavg = np.convolve(data, weights, mode='full')[:len(data)]
    mavg[:periods] = mavg[periods]
    return mavg


def relative_strength(prices, periods=14):
    """
    compute the <periods> period relative strength indicator
    http://stockcharts.com/school/doku.php?\
        id=chart_school:glossary_r#relativestrengthindex
    http://www.investopedia.com/terms/r/rsi.asp
    """
    deltas = np.diff(prices)
    seed = deltas[:periods + 1]
    up = seed[seed >= 0].sum() / periods
    down = -seed[seed < 0].sum() / periods
    ratio = up / down
    rsi = np.zeros_like(prices)
    rsi[:periods] = 100. - 100. / (1. + ratio)

    for i in range(periods, len(prices)):
        delta = deltas[i - 1]  # Cause the diff is 1 shorter

        if delta > 0:
            upval = delta
            downval = 0.
        else:
            upval = 0.
            downval = -delta

        up = (up * (periods - 1) + upval) / periods
        down = (down * (periods - 1) + downval) / periods

        ratio = up / down
        rsi[i] = 100. - 100. / (1. + ratio)

    return rsi


def moving_average_convergence(data, nslow=26, nfast=12):
    """
    compute the MACD (Moving Average Convergence/Divergence)
    using a fast and slow exponential moving avg
    return value is emaslow, emafast, macd which are len(data) arrays
    """
    emaslow = moving_average(data, nslow, method='exponential')
    emafast = moving_average(data, nfast, method='exponential')
    return emaslow, emafast, emafast - emaslow


def cc_annualize_returns(ret_per_period, periods_per_year):
    ''' Compute cumulative compound annualize returns '''
    return math.log(1 + annualized_returns(ret_per_period, periods_per_year))


def annualized_returns(ret_per_period, periods_per_year):
    ''' Could use:
    res1 = averageReturns(Series([ret_per_period]*periods_per_year), \
            period=1, method='net')
    '''
    return pow(1 + ret_per_period, periods_per_year) - 1


def average_returns(data, **kwargs):
    ''' Compute geometric average returns from a returns time serie'''
    average_type = kwargs.get('method', 'net')
    if average_type == 'net':
        relative = 0
    else:
        relative = -1  # gross
    #start = kwargs.get('start', data.index[0])
    #end = kwargs.get('end', data.index[len(data.index) - 1])
    #delta = kwargs.get('delta', data.index[1] - data.index[0])
    period = kwargs.get('period', None)
    if isinstance(period, int):
        pass
    #else:
        #data = reIndexDF(data, start=start, end=end, delta=delta)
        #period = 1
    avg_ret = 1
    for idx in range(len(data.index)):
        if idx % period == 0:
            avg_ret *= (1 + data[idx] + relative)
    return avg_ret - 1


def cc_returns(data, **kwargs):
    ''' Compute cumulative compound returns '''
    start = kwargs.get('start', None)
    end = kwargs.get('end', dt.datetime.today())
    delta = kwargs.get('deltaya', BDay())
    period = kwargs.get('period', None)
    rets = returns(data, method='net', start=start, end=end,
                   delta=delta, period=period)
    return math.log(1 + rets)


# TODO care of dividends
# TODO care of inflation
def returns(data, **kwargs):
    '''
    Compute returns on the given period

    @param data : time serie to process
    @param kwargs.method: gross or simple returns
    @param delta : period betweend two computed returns
    @param start : with end, will return the return betweend this elapsed time
    @param period : delta is the number of lines/periods provided
    @param end : so said
    @param cumulative: compute cumulative returns
    '''
    returns_type = kwargs.get('method', 'net')
    cumulative = kwargs.get('cumulative', False)
    if returns_type == 'net':
        relative = 0
    else:
        relative = 1  # gross
    start = kwargs.get('start', None)
    end = kwargs.get('end', dt.datetime.today())
    #delta = kwargs.get('delta', None)
    period = kwargs.get('period', 1)
    if isinstance(start, dt.datetime):
        return data[end] / data[start] - 1 + relative
    #elif isinstance(delta, pd.DateOffset) or isinstance(delta, dt.timedelta):
        # FIXME timezone problem
        # FIXME reIndexDF is deprecated
        #data = reIndexDF(data, delta=delta)
        #period = 1
    rets_df = data / data.shift(period) - 1 + relative
    if cumulative:
        return rets_df.cumprod()
    return rets_df[1:]


def daily_returns(data, **kwargs):
    ''' re-compute data on a daily basis '''
    relative = kwargs.get('relative', 0)
    return returns(data, delta=BDay(), relative=relative)


def sharpe_ratio(data):
    ''' Compute sharpe from portfolio historical values '''
    # TODO Dataframe handler
    rets = daily_returns(data)
    return (np.mean(rets) / rets.stdev()) * np.sqrt(len(rets))


def high_low_spread(data):
    ''' Compute continue spread on given dataframe every offset period '''
    # TODO: handling the offset period with reindexing or resampling, like:
    # subIndex = df.index[conditions]
    # df = df.reindex(subIndex)
    return data['high'] - data['low']



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Benchmark loader modified to allow live data streaming
  ------------------------------------------------------

  :copyright (c) 2014 Xavier Bruhiere.
  :license: Apache2.0, see LICENSE for more details.
'''


from datetime import datetime
import pandas as pd
from collections import OrderedDict
import zipline.data.loader as zipline


class LiveBenchmark(object):
    def __init__(self, end, frequency='daily', loopback=4):
        self.last_trading_day = end
        self.loopback = loopback
        if frequency == 'minutely':
            self.offset = pd.datetools.Minute()
        elif frequency == 'hourly':
            self.offset = pd.datetools.Hour()
        elif frequency == 'daily':
            self.offset = pd.datetools.Day()
        else:
            raise NotImplementedError()

    def normalize_date(self, test_date):
        ''' Same function as zipline.finance.trading.py'''
        test_date = pd.Timestamp(test_date, tz='UTC')
        return pd.tseries.tools.normalize_date(test_date)

    def surcharge_market_data(self, bm_symbol='^GSPC'):
        bm_bt, tr_bt = zipline.load_market_data(bm_symbol)
        bm_live, tr_live = self._load_live_market_data(bm_symbol)
        bm = bm_bt.append(bm_live)
        tr_bt.update(tr_live)
        return bm, tr_bt

    def _load_live_market_data(self, bm_symbol='^GSPC'):
        #TODO Parametric
        #event_dt = datetime.today().replace(tzinfo=pytz.utc)
        event_dt = self.normalize_date(datetime.now())

        bm_returns, tr_curves = zipline.load_market_data(bm_symbol)

        dates = pd.date_range(event_dt,
                              periods=len(bm_returns))
        #NOTE What is tr_curves['tid'] ?
        #TODO Replace values to detect the fake later
        tr_fake = OrderedDict(sorted(
            ((pd.Timestamp(event_dt + i * self.offset), c)
             for i, c in enumerate(tr_curves.values())),
            key=lambda t: t[0]))

        # NOTE the 1001 code concept is deprecated
        bm_fake = pd.Series([1001] * len(dates), index=dates)
        for i, dt in enumerate(tr_curves.keys()):
            pd.Timestamp(event_dt + i * self.offset)

        return bm_fake, tr_fake



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition specific errors
  -------------------------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''


import dna.errors


class InvalidConfiguration(dna.errors.FactoryError):
    msg = "invalid configuration: {reason} ({config})"


class PortfolioOptimizationFailed(dna.errors.FactoryError):
    msg = """
[{date}] \
Portfolio optimization failed: {reason}, \
processing {data}
""".strip()


class AlgorithmEventFailed(dna.errors.FactoryError):
    msg = """
[{date}] \
algorithm event failed: {reason}, \
processing {data}
""".strip()


class LoadDataFailed(dna.errors.FactoryError):
    msg = "Failed to load data for {sids}: {reason}"


class LoadMarketSchemeFailed(dna.errors.FactoryError):
    msg = "Failed to load market scheme: {reason}"


class LoadContextFailed(dna.errors.FactoryError):
    msg = "Unable to load data from {driver}: {reason}"


class ExchangeIsClosed(dna.errors.FactoryError):
    msg = "{exchange} is closed during {dates}"


class InvalidEngine(dna.errors.FactoryError):
    msg = "Invalid engine {id}: {reason}"


class InvalidDatafeed(dna.errors.FactoryError):
    msg = "Invalid datafeed setup: {reason}"



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Forex access thanks to truefx.com
  ---------------------------------

  :copyright (c) 2014 Xavier Bruhiere.
  :license: Apache2.0, see LICENSE for more details.
'''


import os
import requests
import string
import random
from pandas import DataFrame, Series
import dna.logging

log = dna.logging.logger(__name__)


def _clean_pairs(pairs):
    if not isinstance(pairs, list):
        pairs = [pairs]
    return ','.join(map(str.upper, pairs))


def _fx_mapping(raw_rates):
    ''' Map raw output to clearer labels '''
    return {pair[0].lower(): {
        'timeStamp': pair[1],
        'bid': float(pair[2] + pair[3]),
        'ask': float(pair[4] + pair[5]),
        'high': float(pair[6]),
        'low': float(pair[7])
    } for pair in map(lambda x: x.split(','), raw_rates)}


class TrueFX(object):

    _api_url = 'http://webrates.truefx.com/rates/connect.html'
    _full_snapshot = 'y'
    _output_format = 'csv'

    def __init__(self, credentials='', pairs=[]):
        if not credentials:
            log.info('No credentials provided, inspecting environment')
            credentials = os.environ.get('TRUEFX_API', ':')

        self._user = credentials.split(':')[0]
        self._pwd = credentials.split(':')[1]
        self.state_pairs = _clean_pairs(pairs)
        #self._qualifier = 'ozrates'
        self._qualifier = ''.join(
            random.choice(string.ascii_lowercase) for _ in range(8))

    def connect(self):
        payload = {
            'u': self._user,
            'p': self._pwd,
            'q': self._qualifier,
            'c': self.state_pairs,
            'f': self._output_format,
            's': self._full_snapshot
        }
        auth = requests.get(self._api_url, params=payload)

        if auth.ok:
            log.debug('Truefx authentification successful')
        # Remove '\r\n'
        self._session = auth.content[:-2]
        return auth.ok

    def query_rates(self, pairs=[]):
        ''' Perform a request against truefx data '''
        # If no pairs, TrueFx will use the ones given the last time
        payload = {'id': self._session}
        if pairs:
            payload['c'] = _clean_pairs(pairs)
        response = requests.get(self._api_url, params=payload)

        mapped_data = _fx_mapping(response.content.split('\n')[:-2])
        return Series(mapped_data) if len(mapped_data) == 1 \
            else DataFrame(mapped_data)



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition core
  --------------

  Higher level of Intuition

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

import os
import dna.logging
from intuition import __version__
import intuition.utils as utils
import intuition.api.datafeed as datafeed
from intuition.core.engine import Simulation
import intuition.core.configuration as setup

log = dna.logging.logger(__name__)


def intuition(args):
    '''
    Main simulation wrapper
    Load the configuration, run the engine and return the analyze.
    '''

    # Use the provided context builder to load:
    #   - config: General behavior
    #   - strategy: Modules properties
    #   - market: The universe we will trade on
    with setup.Context(args['context']) as context:

        # Backtest or live engine.
        # Registers configuration and setups data client
        simulation = Simulation()

        # Intuition building blocks
        modules = context['config']['modules']

        # Prepare benchmark, timezone, trading calendar
        simulation.configure_environment(
            context['config']['index'][-1],
            context['market'].benchmark,
            context['market'].timezone
        )

        # Wire togetether modules and initialize them
        simulation.build(args['session'],
                         modules,
                         context['strategy'])

        # Build data generator
        # NOTE How can I use several sources ?
        data = {
            'universe': context['market'],
            'index': context['config']['index']
        }
        # Add user settings
        data.update(context['strategy']['data'])
        # Load backtest and / or live module(s)
        if 'backtest' in modules:
            data['backtest'] = utils.intuition_module(modules['backtest'])
        if 'live' in modules:
            data['live'] = utils.intuition_module(modules['live'])

        # Run the simulation and return an intuition.core.analyzes object
        return simulation(datafeed.HybridDataFactory(**data), args['bot'])


def main():
    # General simulation behavior is defined using command line arguments
    exit_status = 0
    args = setup.parse_commandline()
    loglevel = os.environ.get('LOG', 'warning')
    logfile = setup.logfile(args['session'])
    # Setup structured logs with file and stdout support
    log_setup = dna.logging.setup(
        level=loglevel, show_log=args['showlog'], filename=logfile)

    with log_setup.applicationbound():
        try:
            log.info('intuition v{} ready'.format(__version__),
                     level=loglevel, bot=args['bot'],
                     context=args['context'],
                     session=args['session'])

            analyzes = intuition(args)
            analyzes.build_report(show=args['showlog'])
        except KeyboardInterrupt:
            log.info('Received SIGINT, cleaning...')
        except Exception as error:
            if loglevel == 'debug':
                raise
            log.error('{}: {}'.format(type(error).__name__, str(error)))
            exit_status = 1

        finally:
            log.info('session ended with status {}'.format(exit_status))

    return exit_status



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition results analyzer
  --------------------------

  Wraps session results with convenient analyse methods

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

import pytz
import pandas as pd
import numpy as np
import dna.logging
import dna.debug
import dna.utils
from zipline.data.benchmarks import get_benchmark_returns
from intuition.finance import qstk_get_sharpe_ratio

log = dna.logging.logger(__name__)


class Analyze():
    ''' Handle backtest results and performances measurments '''
    def __init__(self, params, results, metrics, benchmark='^GSPC'):
        # NOTE Temporary
        # Simulation parameters
        self.sim_params = params
        # Final risk measurments as returned by the backtester
        self.results = results
        # Simulation rolling performance
        self.metrics = metrics
        # Market where we traded
        self.benchmark = benchmark

    def build_report(self, timestamp='one_month', show=False):
        # Get daily, cumulative and not, returns of portfolio and benchmark
        # NOTE Temporary fix before intuition would be able to get benchmark
        # data on live trading
        try:
            bm_sym = self.benchmark
            returns_df = self.get_returns(benchmark=bm_sym)
            skip = False
        except:
            log.warn('unable to get benchmark data on live trading for now')
            skip = True

        orders = 0
        for order in self.results.orders:
            orders += len(order)

        final_value = self.results.portfolio_value[-1]
        report = {
            'portfolio': final_value,
            'gain': final_value - self.sim_params.capital_base,
            'orders': orders,
            'pnl_mean': self.results.pnl.mean(),
            'pnl_deviation': self.results.pnl.std(),
        }
        if not skip:
            report['portfolio_perfs'] = returns_df['algo_c_return'][-1] * 100.0
            report['benchmark_perfs'] = \
                returns_df['benchmark_c_return'][-1] * 100.0

        perfs = self.overall_metrics(timestamp)
        for k, v in perfs.iteritems():
            report[k] = v

        # Float values for humans
        for key, value in report.iteritems():
            report[key] = dna.utils.truncate(value, 3)

        # Infinite sharpe ration breaks a lot of tools using this report
        # It usually happens when no transaction were processed
        if np.isinf(report['sharpe']):
            report['sharpe'] = 0.0

        log.info('generated report', report=report)
        if show:
            print
            print(dna.debug.emphasis(report, align=True))
            print
        return report

    def _to_perf_array(self, timestamp, key, length):
        return np.array([self.metrics[timestamp][i][key] for i in length])

    def rolling_performances(self, timestamp='one_month'):
        ''' Filters self.perfs '''
        # TODO Study the impact of month choice
        # TODO Check timestamp in an enumeration
        # TODO Implement other benchmarks for perf computation
        # (zipline issue, maybe expected)

        if self.metrics:
            perfs = {}
            length = range(len(self.metrics[timestamp]))
            index = self._get_index(self.metrics[timestamp])
            perf_keys = self.metrics[timestamp][0].keys()
            perf_keys.pop(perf_keys.index('period_label'))

            perfs['period'] = np.array(
                [pd.datetime.date(date) for date in index])
            for key in perf_keys:
                perfs[key] = self._to_perf_array(timestamp, key, length)
        else:
            # TODO Get it from DB if it exists
            raise NotImplementedError()

        return pd.DataFrame(perfs, index=index)

    def overall_metrics(self, timestamp='one_month', metrics=None):
        '''
        Use zipline results to compute some performance indicators
        '''
        perfs = dict()

        # If no rolling perfs provided, computes it
        if metrics is None:
            metrics = self.rolling_performances(timestamp=timestamp)
        riskfree = np.mean(metrics['treasury_period_return'])

        perfs['sharpe'] = qstk_get_sharpe_ratio(
            metrics['algorithm_period_return'].values, risk_free=riskfree)
        perfs['algorithm_period_return'] = (
            ((metrics['algorithm_period_return'] + 1).cumprod()) - 1)[-1]
        perfs['max_drawdown'] = max(metrics['max_drawdown'])
        perfs['algo_volatility'] = np.mean(metrics['algo_volatility'])
        perfs['beta'] = np.mean(metrics['beta'])
        perfs['alpha'] = np.mean(metrics['alpha'])
        perfs['benchmark_period_return'] = (
            ((metrics['benchmark_period_return'] + 1).cumprod()) - 1)[-1]

        return perfs

    def get_returns(self, benchmark=''):
        returns = {}

        if benchmark:
            try:
                benchmark_data = (
                    get_benchmark_returns(benchmark,
                                          self.results.index[0],
                                          self.results.index[-1]))
            except Exception as e:
                raise KeyError(e)
        else:
            # TODO Automatic detection given exchange market ?
            raise NotImplementedError()

        # NOTE Could be more efficient. But len(benchmark_data.date) !=
        # len(self.results.returns.index). Maybe because of different markets
        dates = pd.DatetimeIndex([d.date for d in benchmark_data])

        returns['benchmark_return'] = pd.Series(
            [d.returns for d in benchmark_data], index=dates)
        returns['benchmark_c_return'] = (
            (returns['benchmark_return'] + 1).cumprod()) - 1
        returns['algo_return'] = pd.Series(
            self.results.returns.values, index=dates)
        returns['algo_c_return'] = pd.Series(
            ((self.results.returns.values + 1).cumprod()) - 1, index=dates)

        df = pd.DataFrame(returns, index=dates)

        if benchmark is None:
            df = df.drop(['benchmark_return', 'benchmark_c_return'], axis=1)
        return df

    def _get_index(self, perfs):
        # NOTE No frequency infos or just period number ?
        start = pytz.utc.localize(pd.datetime.strptime(
            perfs[0]['period_label'] + '-01', '%Y-%m-%d'))
        end = pytz.utc.localize(pd.datetime.strptime(
            perfs[-1]['period_label'] + '-01', '%Y-%m-%d'))
        return pd.date_range(start - pd.datetools.BDay(10),
                             end,
                             freq=pd.datetools.MonthBegin())



'''
  Intuition
  ---------

  Intuition is an engine, some building bricks and a set of tools meant to let
  you efficiently and intuitively make your own automated quantitative trading
  system. It is designed to let traders, developers and scientists explore,
  improve and deploy market technical hacks.

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

__project__ = 'intuition'
__author__ = 'Xavier Bruhiere'
__copyright__ = 'Xavier Bruhiere'
__licence__ = 'Apache 2.0'
__version__ = '0.4.2'



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition Portfolio manager api
  -------------------------------

  Portfolio manager factory. Assets allocation strategies should inherit from
  this class.

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''


import abc
import dna.logging
from intuition.errors import PortfolioOptimizationFailed


def _remove_useless_orders(alloc):
    ''' Pop from the orderbook orders of 0 values '''
    return {sid: value for sid, value in alloc.iteritems() if value != 0}


class PortfolioFactory():
    '''
    Manages portfolio during simulation, and stays aware of the situation
    through the update() method.

    User strategies call it with a dictionnnary of detected opportunities (i.e.
    buy or sell signals).  Then the optimize function computes assets
    allocation, returning a dictionnary of symbols with their weigths or amount
    to reallocate.

    {'buy': zipline.Positions}    __________________________      _____________
    signals                   --> |                         | -> |            |
                                  | trade_signals_handler() |    | optimize() |
    orders  {'google': 34}    <-- |_________________________| <- |____________|

    This is abstract class, inheretid class will eventally overwrite optmize()
    to expose their own asset allocation strategy.
    '''

    __metaclass__ = abc.ABCMeta

    # Zipline portfolio object, updated during simulation with self.date
    portfolio = None
    perfs = None
    date = None

    # TODO Add in the constructor or setup parameters some general settings
    # like maximum weights, positions, frequency, ...
    # TODO Remove stocks with 0 to allocate
    # NOTE Regarding portfolio constraints: from a set of user-defined
    # parameters, a unique set should be constructed. Then the solution
    # provided by optimize function would have to be a subset of it. (classic
    # mathematical solution) Finally it should be defined how to handle
    # non-correct solutions

    def __init__(self, properties=None):
        self.log = dna.logging.logger(__name__)
        # Other parameters are used in user's optimize() method
        self.properties = properties or {}
        self.initialize(self.properties)

    def initialize(self, properties):
        ''' Users should overwrite this method '''
        pass

    @abc.abstractmethod
    def optimize(self, to_buy={}, to_sell={}):
        ''' Users should overwrite this method '''
        pass

    def update(self, portfolio, date, perfs=None):
        '''
        Actualizes the portfolio universe with the alog state
        '''
        # Make the manager aware of current simulation
        self.portfolio = portfolio
        self.perfs = perfs
        self.date = date

    def trade_signals_handler(self, signals):
        '''
        Process buy and sell signals from the simulation
        '''
        buy_sig, sell_sig = signals.get('buy', {}), signals.get('sell', {})
        if buy_sig or sell_sig:
            # Compute optimal portfolio allocation, Using user defined function
            try:
                alloc, e_ret, e_risk = self.optimize(buy_sig, sell_sig)
            except Exception, error:
                raise PortfolioOptimizationFailed(
                    reason=error, date=self.date, data=signals)

        return _remove_useless_orders(alloc)

    def advise(self, **kwargs):
        '''
        General parameters or user settings
        (maw_weigth, max_assets, max_frequency, commission cost)
        '''
        for name, value in kwargs.iteritems():
            self.properties[name] = value



#
#  ystockquote : Python module - retrieve stock quote data from Yahoo Finance
#
#  Copyright (c) 2007,2008,2013 Corey Goldberg (cgoldberg@gmail.com)
#
#  license: GNU LGPL
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  Requires: Python 2.7/3.3+


__version__ = '0.2.5dev'

from bs4 import BeautifulSoup
try:
    # py3
    from urllib.request import Request, urlopen
    from urllib.parse import urlencode
except ImportError:
    # py2
    from urllib2 import Request, urlopen
    from urllib import urlencode


def _request(symbol, stat):
    url = 'http://finance.yahoo.com/d/quotes.csv?s=%s&f=%s' % (symbol, stat)
    req = Request(url)
    resp = urlopen(req)
    content = resp.read().decode().strip()
    return content


def get_all(symbol):
    """
    Get all available quote data for the given ticker symbol.

    Returns a dictionary.
    """
    ids = \
        'ydb2r1b3qpoc1d1cd2c6t1k2p2c8m5c3m6gm7hm8k1m3lm4l1t8w1g1w4g3p' \
        '1g4mg5m2g6kvjj1j5j3k4f6j6nk5n4ws1xj2va5b6k3t7a2t615l2el3e7v1' \
        'e8v7e9s6b4j4p5p6rr2r5r6r7s7'
    values = _request(symbol, ids).split(',')
    return dict(
        dividend_yield=values[0],
        dividend_per_share=values[1],
        ask_realtime=values[2],
        dividend_pay_date=values[3],
        bid_realtime=values[4],
        ex_dividend_date=values[5],
        previous_close=values[6],
        today_open=values[7],
        change=values[8],
        last_trade_date=values[9],
        change_percent_change=values[10],
        trade_date=values[11],
        change_realtime=values[12],
        last_trade_time=values[13],
        change_percent_realtime=values[14],
        change_percent=values[15],
        after_hours_change_realtime=values[16],
        change_200_sma=values[17],
        todays_low=values[18],
        change_50_sma=values[19],
        todays_high=values[20],
        percent_change_50_sma=values[21],
        last_trade_realtime_time=values[22],
        fifty_sma=values[23],
        last_trade_time_plus=values[24],
        twohundred_sma=values[25],
        last_trade_price=values[26],
        one_year_target=values[27],
        todays_value_change=values[28],
        holdings_gain_percent=values[29],
        todays_value_change_realtime=values[30],
        annualized_gain=values[31],
        price_paid=values[32],
        holdings_gain=values[33],
        todays_range=values[34],
        holdings_gain_percent_realtime=values[35],
        todays_range_realtime=values[36],
        holdings_gain_realtime=values[37],
        fiftytwo_week_high=values[38],
        more_info=values[39],
        fiftytwo_week_low=values[40],
        market_cap=values[41],
        change_from_52_week_low=values[42],
        market_cap_realtime=values[43],
        change_from_52_week_high=values[44],
        float_shares=values[45],
        percent_change_from_52_week_low=values[46],
        company_name=values[47],
        percent_change_from_52_week_high=values[48],
        notes=values[49],
        fiftytwo_week_range=values[50],
        shares_owned=values[51],
        stock_exchange=values[52],
        shares_outstanding=values[53],
        volume=values[54],
        ask_size=values[55],
        bid_size=values[56],
        last_trade_size=values[57],
        ticker_trend=values[58],
        average_daily_volume=values[59],
        trade_links=values[60],
        order_book_realtime=values[61],
        high_limit=values[62],
        eps=values[63],
        low_limit=values[64],
        eps_estimate_current_year=values[65],
        holdings_value=values[66],
        eps_estimate_next_year=values[67],
        holdings_value_realtime=values[68],
        eps_estimate_next_quarter=values[69],
        revenue=values[70],
        book_value=values[71],
        ebitda=values[72],
        price_sales=values[73],
        price_book=values[74],
        pe=values[75],
        pe_realtime=values[76],
        peg=values[77],
        price_eps_estimate_current_year=values[78],
        price_eps_estimate_next_year=values[79],
        short_ratio=values[80],
    )


def get_dividend_yield(symbol):
    return _request(symbol, 'y')


def get_dividend_per_share(symbol):
    return _request(symbol, 'd')


def get_ask_realtime(symbol):
    return _request(symbol, 'b2')


def get_dividend_pay_date(symbol):
    return _request(symbol, 'r1')


def get_bid_realtime(symbol):
    return _request(symbol, 'b3')


def get_ex_dividend_date(symbol):
    return _request(symbol, 'q')


def get_previous_close(symbol):
    return _request(symbol, 'p')


def get_today_open(symbol):
    return _request(symbol, 'o')


def get_change(symbol):
    return _request(symbol, 'c1')


def get_last_trade_date(symbol):
    return _request(symbol, 'd1')


def get_change_percent_change(symbol):
    return _request(symbol, 'c')


def get_trade_date(symbol):
    return _request(symbol, 'd2')


def get_change_realtime(symbol):
    return _request(symbol, 'c6')


def get_last_trade_time(symbol):
    return _request(symbol, 't1')


def get_change_percent_realtime(symbol):
    return _request(symbol, 'k2')


def get_change_percent(symbol):
    return _request(symbol, 'p2')


def get_after_hours_change(symbol):
    return _request(symbol, 'c8')


def get_change_200_sma(symbol):
    return _request(symbol, 'm5')


def get_commission(symbol):
    return _request(symbol, 'c3')


def get_percent_change_200_sma(symbol):
    return _request(symbol, 'm6')


def get_todays_low(symbol):
    return _request(symbol, 'g')


def get_change_50_sma(symbol):
    return _request(symbol, 'm7')


def get_todays_high(symbol):
    return _request(symbol, 'h')


def get_percent_change_50_sma(symbol):
    return _request(symbol, 'm8')


def get_last_trade_realtime_time(symbol):
    return _request(symbol, 'k1')


def get_50_sma(symbol):
    return _request(symbol, 'm3')


def get_last_trade_time_plus(symbol):
    return _request(symbol, 'l')


def get_200_sma(symbol):
    return _request(symbol, 'm4')


def get_last_trade_price(symbol):
    return _request(symbol, 'l1')


def get_1_year_target(symbol):
    return _request(symbol, 't8')


def get_todays_value_change(symbol):
    return _request(symbol, 'w1')


def get_holdings_gain_percent(symbol):
    return _request(symbol, 'g1')


def get_todays_value_change_realtime(symbol):
    return _request(symbol, 'w4')


def get_annualized_gain(symbol):
    return _request(symbol, 'g3')


def get_price_paid(symbol):
    return _request(symbol, 'p1')


def get_holdings_gain(symbol):
    return _request(symbol, 'g4')


def get_todays_range(symbol):
    return _request(symbol, 'm')


def get_holdings_gain_percent_realtime(symbol):
    return _request(symbol, 'g5')


def get_todays_range_realtime(symbol):
    return _request(symbol, 'm2')


def get_holdings_gain_realtime(symbol):
    return _request(symbol, 'g6')


def get_52_week_high(symbol):
    return _request(symbol, 'k')


def get_more_info(symbol):
    return _request(symbol, 'v')


def get_52_week_low(symbol):
    return _request(symbol, 'j')


def get_market_cap(symbol):
    return _request(symbol, 'j1')


def get_change_from_52_week_low(symbol):
    return _request(symbol, 'j5')


def get_market_cap_realtime(symbol):
    return _request(symbol, 'j3')


def get_change_from_52_week_high(symbol):
    return _request(symbol, 'k4')


def get_float_shares(symbol):
    return _request(symbol, 'f6')


def get_percent_change_from_52_week_low(symbol):
    return _request(symbol, 'j6')


def get_company_name(symbol):
    return _request(symbol, 'n')


def get_percent_change_from_52_week_high(symbol):
    return _request(symbol, 'k5')


def get_notes(symbol):
    return _request(symbol, 'n4')


def get_52_week_range(symbol):
    return _request(symbol, 'w')


def get_shares_owned(symbol):
    return _request(symbol, 's1')


def get_stock_exchange(symbol):
    return _request(symbol, 'x')


def get_shares_outstanding(symbol):
    return _request(symbol, 'j2')


def get_volume(symbol):
    return _request(symbol, 'v')


def get_ask_size(symbol):
    return _request(symbol, 'a5')


def get_bid_size(symbol):
    return _request(symbol, 'b6')


def get_last_trade_size(symbol):
    return _request(symbol, 'k3')


def get_ticker_trend(symbol):
    return _request(symbol, 't7')


def get_average_daily_volume(symbol):
    return _request(symbol, 'a2')


def get_trade_links(symbol):
    return _request(symbol, 't6')


def get_order_book_realtime(symbol):
    return _request(symbol, 'i5')


def get_high_limit(symbol):
    return _request(symbol, 'l2')


def get_eps(symbol):
    return _request(symbol, 'e')


def get_low_limit(symbol):
    return _request(symbol, 'l3')


def get_eps_estimate_current_year(symbol):
    return _request(symbol, 'e7')


def get_holdings_value(symbol):
    return _request(symbol, 'v1')


def get_eps_estimate_next_year(symbol):
    return _request(symbol, 'e8')


def get_holdings_value_realtime(symbol):
    return _request(symbol, 'v7')


def get_eps_estimate_next_quarter(symbol):
    return _request(symbol, 'e9')


def get_revenue(symbol):
    return _request(symbol, 's6')


def get_book_value(symbol):
    return _request(symbol, 'b4')


def get_ebitda(symbol):
    return _request(symbol, 'j4')


def get_price_sales(symbol):
    return _request(symbol, 'p5')


def get_price_book(symbol):
    return _request(symbol, 'p6')


def get_pe(symbol):
    return _request(symbol, 'r')


def get_pe_realtime(symbol):
    return _request(symbol, 'r2')


def get_peg(symbol):
    return _request(symbol, 'r5')


def get_price_eps_estimate_current_year(symbol):
    return _request(symbol, 'r6')


def get_price_eps_estimate_next_year(symbol):
    return _request(symbol, 'r7')


def get_short_ratio(symbol):
    return _request(symbol, 's7')


def get_sector(symbol):
    '''
    Uses BeautifulSoup to scrape stock sector from Yahoo! Finance website
    '''
    url = 'http://finance.yahoo.com/q/pr?s=%s+Profile' % symbol
    soup = BeautifulSoup(urlopen(url).read())
    try:
        sector = soup.find('td', text='Sector:').\
            find_next_sibling().string.encode('utf-8')
    except:
        sector = ''
    return sector


def get_industry(symbol):
    '''
    Uses BeautifulSoup to scrape stock industry from Yahoo! Finance website
    '''
    url = 'http://finance.yahoo.com/q/pr?s=%s+Profile' % symbol
    soup = BeautifulSoup(urlopen(url).read())
    try:
        industry = soup.find('td', text='Industry:').\
            find_next_sibling().string.encode('utf-8')
    except:
        industry = ''
    return industry


def get_type(symbol):
    '''
    Uses BeautifulSoup to scrape symbol category from Yahoo! Finance website
    '''
    url = 'http://finance.yahoo.com/q/pr?s=%s+Profile' % symbol
    soup = BeautifulSoup(urlopen(url).read())
    if soup.find('span', text='Business Summary'):
        return 'Stock'
    elif soup.find('span', text='Fund Summary'):
        asset_type = 'Fund'
    elif symbol.find('^') == 0:
        asset_type = 'Index'
    else:
        pass
    return asset_type


def get_indices(symbol):
    url = 'http://finance.yahoo.com/q/pr?s=%s+Profile' % symbol
    soup = BeautifulSoup(urlopen(url).read())
    indices = []
    try:
        tmp_idx = soup.find('td', text='Index Membership:').\
            find_next_sibling().findAll('a')
        for idx in tmp_idx:
            indices.append(idx.text)
    except:
        pass
    return ','.join(indices)


def get_historical_prices(symbol, start_date, end_date):
    """
    Get historical prices for the given ticker symbol.
    Date format is 'YYYY-MM-DD'

    Returns a nested dictionary (dict of dicts).
    outer dict keys are dates ('YYYY-MM-DD')
    """
    params = urlencode({
        's': symbol,
        'a': int(start_date[5:7]) - 1,
        'b': int(start_date[8:10]),
        'c': int(start_date[0:4]),
        'd': int(end_date[5:7]) - 1,
        'e': int(end_date[8:10]),
        'f': int(end_date[0:4]),
        'g': 'd',
        'ignore': '.csv',
    })
    url = 'http://ichart.yahoo.com/table.csv?%s' % params
    req = Request(url)
    resp = urlopen(req)
    content = str(resp.read().decode('utf-8').strip())
    daily_data = content.splitlines()
    hist_dict = dict()
    keys = daily_data[0].split(',')
    for day in daily_data[1:]:
        day_data = day.split(',')
        date = day_data[0]
        hist_dict[date] = \
            {keys[1]: day_data[1],
             keys[2]: day_data[2],
             keys[3]: day_data[3],
             keys[4]: day_data[4],
             keys[5]: day_data[5],
             keys[6]: day_data[6]}
    return hist_dict



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Packaging
  ---------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

import os
from glob import glob
from setuptools import setup, find_packages
from intuition import (
    __version__, __author__, __licence__, __project__
)


def get_requirements():
    with open('./requirements.txt') as requirements:
        # Avoid github based requirements and replace them
        deps = requirements.read().split('\n')[:-2]
        deps.append('zipline>=0.5.11.dev')
    return deps


requires = [
    'beautifulsoup4>=4.3.2',
    'blist>=1.3.6',
    'Cython>=0.20.1',
    'ystockquote',
    'numpy>=1.8.1',
    'schematics>=0.9-4',
    'schema>=0.3.1',
    'python-dateutil>=2.2',
    'pytz>=2014.3',
    'PyYAML>=3.11',
    'Quandl>=1.9.7',
    'dna>=0.0.4',
    'requests>=2.3.0',
    'six>=1.6.1',
    'zipline>=0.5.9',
    'pandas>=0.13.1'
]


def long_description():
    try:
        #with codecs.open(readme, encoding='utf8') as f:
        with open('README.md') as f:
            return f.read()
    except IOError:
        return "failed to read README.md"


setup(
    name=__project__,
    version=__version__,
    description='A trading system building blocks',
    author=__author__,
    author_email='xavier.bruhiere@gmail.com',
    packages=find_packages(),
    long_description=long_description(),
    license=__licence__,
    install_requires=requires,
    url="https://github.com/hackliff/intuition",
    entry_points={
        'console_scripts': [
            'intuition = intuition.__main__:main',
        ],
    },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Operating System :: OS Independent',
        'Intended Audience :: Science/Research',
        'Topic :: Office/Business :: Financial',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: System :: Distributed Computing',
    ],
    data_files=[
        (os.path.expanduser('~/.intuition/data'), glob('data/*')),
        (os.path.expanduser('~/.intuition/logs'), glob('./MANIFEST.in'))
    ],
    dependency_links=[
        'http://github.com/quantopian/zipline/tarball/master#egg=zipline-0.5.11.dev'
    ]
)



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition misc utilities
  ------------------------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''


import time
import datetime as dt
import pytz
import pandas as pd
import dna.utils
from dna.time_utils import normalize_date_format


def is_live(current_date):
    ''' True if the given date is in the (utc) future '''
    return (current_date > pd.datetime.now(pytz.utc))


def next_tick(date, interval=15):
    '''
    Only return when we reach given datetime
    '''
    # Intuition works with utc dates, conversion are made for I/O
    now = dt.datetime.now(pytz.utc)
    live = False
    # Sleep until we reach the given date
    while now < date:
        time.sleep(interval)
        # Update current time
        now = dt.datetime.now(pytz.utc)
        # Since we're here, we waited a future date, so this is live trading
        live = True
    return live


def intuition_module(location):
    ''' Build the module path and import it '''
    path = location.split('.')
    # Get the last field, i.e. the object name in the file
    obj = path.pop(-1)
    return dna.utils.dynamic_import('.'.join(path), obj)


# NOTE Without the freq param, their might be some smart refactor to do here
def build_trading_timeline(start, end):
    ''' Build the daily-based index we will trade on '''
    EMPTY_DATES = pd.date_range('2000/01/01', periods=0, tz=pytz.utc)
    now = dt.datetime.now(tz=pytz.utc)

    if not start:
        if not end:
            # Live trading until the end of the day
            bt_dates = EMPTY_DATES
            live_dates = pd.date_range(
                start=now,
                end=normalize_date_format('23h59'))
        else:
            end = normalize_date_format(end)
            if end < now:
                # Backtesting since a year before end
                bt_dates = pd.date_range(
                    start=end - 360 * pd.datetools.day,
                    end=end)
                live_dates = EMPTY_DATES
            elif end > now:
                # Live trading from now to end
                bt_dates = EMPTY_DATES
                live_dates = pd.date_range(start=now, end=end)
    else:
        start = normalize_date_format(start)
        if start < now:
            if not end:
                # Backtest for a year or until now
                end = start + 360 * pd.datetools.day
                if end > now:
                    end = now - pd.datetools.day
                live_dates = EMPTY_DATES
                bt_dates = pd.date_range(
                    start=start, end=end)
            else:
                end = normalize_date_format(end)
                if end < now:
                    # Nothing to do, backtest from start to end
                    live_dates = EMPTY_DATES
                    bt_dates = pd.date_range(start=start, end=end)
                elif end > now:
                    # Hybrid timeline, backtest from start to end, live
                    # trade from now to end
                    bt_dates = pd.date_range(
                        start=start, end=now - pd.datetools.day)

                    live_dates = pd.date_range(start=now, end=end)
        elif start > now:
            if not end:
                # Live trading from start to the end of the day
                bt_dates = EMPTY_DATES
                live_dates = pd.date_range(
                    start=start,
                    end=normalize_date_format('23h59'))
            else:
                # Live trading from start to end
                end = normalize_date_format(end)
                bt_dates = EMPTY_DATES
                live_dates = pd.date_range(start=start, end=end)

    return bt_dates + live_dates



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition remote data
  ---------------------

  Provides functions to access various trading data accross internet wires

  :copyright (c) 2014 Xavier Bruhiere.
  :license: Apache2.0, see LICENSE for more details.
'''

import json
import requests
import pandas as pd
from pandas.io.data import DataReader, get_quote_yahoo
import dna.logging
from intuition.constants import FINANCE_URLS
import intuition.data.ystockquote as ystockquote
import intuition.data.utils as utils

log = dna.logging.logger(__name__)


def historical_pandas_yahoo(symbol, source='yahoo', start=None, end=None):
    '''
    Fetch from yahoo! finance historical quotes
    '''
    # NOTE Panel for multiple symbols ?
    # NOTE Adj Close column name not cool (a space)
    return DataReader(symbol, source, start=start, end=end)


@utils.invert_dataframe_axis
def snapshot_yahoo_pandas(symbols):
    '''
    Get a simple snapshot from yahoo, return dataframe
    __________________________________________________
    Return
        pandas.DataFrame with symbols as index
        and columns = [change_pct, time, last, short_ratio, PE]
    '''
    if isinstance(symbols, str):
        symbols = [symbols]
    # TODO lower() columns
    data = get_quote_yahoo(symbols)
    data.index = [str(sid).lower() for sid in data.index]
    return data


# NOTE Can use symbol with market: 'goog:nasdaq', any difference ?
@utils.fractionate_request
@utils.use_google_symbol
def snapshot_google(symbols):
    payload = {'client': 'ig', 'q': ','.join(symbols)}
    response = requests.get(FINANCE_URLS['snapshot_google_light'],
                            params=payload)
    # FIXME Some errors are not detected
    try:
        json_infos = json.loads(response.text[3:], encoding='utf-8')
    except ValueError:
        log.error("no reliable data found")
        return pd.DataFrame()

    snapshot = {}
    for i, quote in enumerate(json_infos):
        if quote['t'].lower() in map(str.lower, symbols):
            snapshot[symbols[i]] = utils.apply_mapping(
                quote, google_light_mapping)
        else:
            log.warning('Unknown symbol {}, ignoring...'.format(
                quote['t']))

    return pd.DataFrame(snapshot)


@property
def google_light_mapping():
    return {
        'change': (str, 'c'),
        'str_change': (str, 'ccol'),
        'perc_change': (float, 'cp'),
        'exchange': (str, 'e'),
        'id': (int, 'id'),
        'price': (str, 'l'),
        'last_price': (lambda x: x, 'l_cur'),
        'date': (str, 'lt'),
        'time': (str, 'ltt'),
        's': (int, 's'),
        'symbol': (str, 't'),
    }


def lookup_symbol(company):
    ''' Fetch company info
    >>> lookup_symbol("Apple Inc.")
          {u'exch': u'NMS',
           u'market': u'NASDAQ',
           u'name': u'Apple Inc.',
           u'symbol': u'AAPL',
           u'type': u'Equity'}
    '''
    request = requests.get(FINANCE_URLS['info_lookup'].format(company))
    return json.loads(request.text[39:-1])["ResultSet"]["Result"] \
        if request.ok else {'error': request.reason}


def fill_stock_metadata(sid):
    ''' Fetch extra informations about a stock, from yahoo '''
    metadata = {}
    try:
        metadata['name'] = ystockquote.get_company_name(sid)
    except Exception, error:
        log.error(error)
        log.warning('sometimes it just times out, retry later')
        return {}

    metadata['exchange'] = ystockquote.get_stock_exchange(sid)
    if metadata['exchange'].find('Nasdaq') > 0:
        # It returns NasdaqNM
        metadata['exchange'] = 'Nasdaq'
    metadata['type'] = ystockquote.get_type(sid)
    metadata['revenue'] = ystockquote.get_revenue(sid)
    # FIXME Almost never found
    # metadata['index'] = ystockquote.get_indices(sid)
    metadata['sector'] = ystockquote.get_sector(sid)
    metadata['industry'] = ystockquote.get_industry(sid)

    log.debug('successfully fetched metadata about {}'.format(sid))
    log.debug(dna.debug.emphasis(metadata))
    return metadata



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition Data source api
  -------------------------

  Data generator motherboard for Intuition. It uses backtest and live data
  sources to build the datafeed of the algorithm.

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

#import pytz
#import datetime as dt
import pandas as pd
import dna.logging
from zipline.sources.data_source import DataSource
from zipline.gens.utils import hash_args
import intuition.utils as utils
from intuition.errors import LoadDataFailed, InvalidDatafeed
import intuition.data.universe as universe


def _build_safe_event(event, date, sid):
    event.update({
        'dt': date,
        'sid': sid,
        'volume': event.get('volume', 1000)
    })
    return event


def _check_data_modules(backtest, live, start, end):
    # TODO Fails if the class has no get_data method
    if not backtest and not live:
        raise InvalidDatafeed(
            reason='provide at least a backtest or a live data module')

    if not backtest and not utils.is_live(start):
        raise InvalidDatafeed(
            reason='no backtest data source provided for past dates')

    if not live and utils.is_live(end):
        raise InvalidDatafeed(
            reason='no live data source provided for futur dates')

    return True


class HybridDataFactory(DataSource):
    '''
    Surcharge of zipline.DataSource, switching automatically between live
    stream and backtest sources
    '''

    backtest = None
    live = None
    _is_live = False

    def __init__(self, **kwargs):
        # TODO Use alternatives to `index` and `universe` objects
        self.log = dna.logging.logger(__name__)

        if 'index' not in kwargs or 'universe' not in kwargs:
            raise InvalidDatafeed(
                reason='you must provide a universe and an index')
        if not isinstance(kwargs.get('index'),
                          pd.tseries.index.DatetimeIndex):
            raise InvalidDatafeed(reason='you must provide a valid time index')

        # Unpack config dictionary with default values.
        self.sids = kwargs['universe'].sids
        self.index = kwargs['index']
        self.start = self.index[0]
        self.end = self.index[-1]

        #self.frequency = float(kwargs.get('frequency', 14))
        self.frequency = kwargs.get('frequency', 'at opening')
        self.market_open = kwargs['universe'].open
        self.market_close = kwargs['universe'].close
        self.market_timezone = kwargs['universe'].timezone

        if 'backtest' in kwargs:
            self.backtest = kwargs['backtest'](self.sids, kwargs)
        if 'live' in kwargs:
            self.live = kwargs['live'](self.sids, kwargs)

        _check_data_modules(self.backtest, self.live, self.start, self.end)

        # Hash_value for downstream sorting.
        self.arg_string = hash_args(**kwargs)
        self._raw_data = None

    @property
    def mapping(self):
        if self._is_live:
            return self.live.mapping
        else:
            return self.backtest.mapping

    def _get_backtest_data(self):
        # The first date is usually a few seconds before now,
        # so we compare to the next one
        if self.backtest and not utils.is_live(self.index[1]):
            try:
                bt_data = self.backtest.get_data(
                    self.sids, self.start, self.end)
            except Exception as error:
                raise LoadDataFailed(sids=self.sids, reason=error)
        else:
            bt_data = None
        return bt_data

    def _agnostic_get_data_at(self, date, data):
        dated_data = pd.DataFrame()
        n_axes = len(data.axes)

        if self._is_live:
            try:
                dated_data = self.live.get_data(self.sids)
            except Exception as error:
                raise LoadDataFailed(sids=self.sids, reason=error)

            if isinstance(dated_data, pd.Series):
                dated_data = pd.DataFrame(
                    {sid: {'price': price}
                     for sid, price in dated_data.iterkv()})

        else:
            #midnight_date = date.replace(hour=0, minute=0)
            midnight_date = pd.datetools.normalize_date(date)
            if n_axes == 2:
                if midnight_date in data.index:
                    dated_data = pd.DataFrame(
                        {sid: {'price': price}
                         for sid, price in data.ix[midnight_date].iterkv()})

            elif n_axes == 3:
                if midnight_date in data.major_axis:
                    dated_data = data.major_xs(midnight_date)

            else:
                raise TypeError('only dataframe and panel are supported')
        return dated_data

    def raw_data_gen(self):
        bt_data = self._get_backtest_data()

        for date in self.index:
            backtest_is_done = False
            #date = date.replace(hour=self.market_open.hour,
            open_hour = date.replace(hour=self.market_open.hour,
                                     minute=self.market_open.minute)
            close_hour = date.replace(hour=self.market_close.hour,
                                      minute=self.market_close.minute)

            tick_ = universe.Tick(
                start=open_hour, end=close_hour, tz=self.market_timezone)
            tick_.parse(self.frequency)

            # Trade until the end of the trading day
            for date in tick_.tack:
                # Set to opening of the market
                self.log.debug('--> next tick {}'.format(date))
                # NOTE Make _is_live a property ?
                self._is_live = utils.next_tick(date)

                data = self._agnostic_get_data_at(date, bt_data)
                if not data.empty:
                    for sid, series in data.iterkv():
                        if backtest_is_done and not self._is_live:
                            # TODO Use previous and next data to extrapolate
                            # random values
                            self.log.debug('extrapoling intraday data')

                        yield _build_safe_event(series.to_dict(), date, sid)
                    backtest_is_done = True

    @property
    def instance_hash(self):
        return self.arg_string

    @property
    def raw_data(self):
        if not self._raw_data:
            self._raw_data = self.raw_data_gen()
        return self._raw_data



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Quandl.com access
  -----------------

  :copyright (c) 2014 Xavier Bruhiere.
  :license: Apache2.0, see LICENSE for more details.
'''


import pandas as pd
import pytz
import os
import dna.logging
import Quandl
import intuition.data.utils as utils

log = dna.logging.logger(__name__)


# http://www.quandl.com/help/api-for-bitcoin-data
BITCOIN_CODE = 'BITCOIN/{market}{currency}'
YAHOO_PARIS_CODE = 'YAHOO/PA_{symbol}'


def _build_quandl_code(symbol):
    dot_pos = symbol.find('.')
    slash_pos = symbol.find('/')
    if dot_pos > 0:
        market = symbol[dot_pos + 1:]
        provider = 'YAHOO'
        symbol = symbol[:dot_pos]
        code = '{}_{}'.format(market, symbol)
    else:
        if slash_pos > 0:
            pair = symbol.split('/')
            provider = 'QUANDL'
            code = '{}{}'.format(pair[0], pair[1])
        else:
            market = 'NASDAQ'
            provider = 'GOOG'
            code = '{}_{}'.format(market, symbol)
    return '{}/{}'.format(provider, code).upper()


def use_quandl_symbols(fct):
    def decorator(symbols, **kwargs):
        if not isinstance(symbols, list):
            symbols = [symbols]
        quandl_symbols = map(_build_quandl_code, symbols)
        raw_data = fct(quandl_symbols, **kwargs)

        data = {}
        for sid in symbols:
            # Filter out other sids
            df_ = raw_data.filter(
                #regex='YAHOO.*_{} -.*'.format(utils.clean_sid(sid).upper())
                regex='.*_{} -.*'.format(utils.clean_sid(sid).upper())
            )
            # Remove Quandl code and useless spaces
            df_.columns = map(
                lambda x: x.replace(' ', '_').lower().split('_-_')[-1],
                df_.columns
            )
            if 'not_found' not in df_.columns:
                data[sid] = df_
            else:
                log.warning('no data was returned for ' + sid)
        return data
    return decorator


@utils.fractionate_request
@use_quandl_symbols
def download(codes, **kwargs):
    try:
        data = Quandl.get(codes, **kwargs)
        data.index = data.index.tz_localize(pytz.utc)
    except Exception, error:
        log.error('unable to fetch {}: {}'.format(codes, error))
        data = pd.DataFrame()

    return data


# NOTE With last simplifications, an object seems useless ?
class DataQuandl(object):
    ''' Quandl.com as datasource '''

    def __init__(self, api_key=None):
        self.api_key = api_key or os.environ.get('QUANDL_API_KEY', None)

    def fetch(self, codes, **kwargs):
        ''' Quandl entry point in datafeed object '''
        log.debug('fetching QuanDL data (%s)' % codes)

        # Harmonization: Quandl call start trim_start
        if 'start' in kwargs:
            kwargs['trim_start'] = kwargs.pop('start')
        if 'end' in kwargs:
            kwargs['trim_end'] = kwargs.pop('end')

        return download(codes, authtoken=self.api_key, **kwargs)



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition alogirthm api
  -----------------------

  Trading algorithms factory. Everything that can extend in a useful way
  zipline.TradingAlgorithm goes here. Future algorithms should inherit from
  this class.

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''


import abc
import datetime as dt
import zipline.algorithm
import intuition.errors


# pylint: disable=R0921
class TradingFactory(zipline.algorithm.TradingAlgorithm):
    '''
    Intuition surcharge of main zipline class, but fully compatible.  Its role
    is slightly different : the user will eventually expose an event() method
    meant to return buy and sell signals, processed, optionnaly, further by the
    portfolio strategy. However it remains fully compatible with zipline method
    (i.e. it's stil possible to execute orders within the event method and not
    consider any portfolio strategy aside)
    '''

    __metaclass__ = abc.ABCMeta

    def __init__(self, identity='johndoe', properties=None):
        # Attributes initialization
        self.sids = []
        self.middlewares = []
        self.auto = False
        self.manager = None
        self.initialized = False
        self.orderbook = {}

        # User customized attributes
        safe_properties = properties or {}
        self.identity = identity
        self.realworld = safe_properties.get('realworld')
        zipline.algorithm.TradingAlgorithm.__init__(
            self, properties=safe_properties)

    def _is_interactive(self):
        ''' Prevent middlewares and orders to work outside live mode '''
        return not (self.realworld and
                    dt.date.today() > self.datetime.date())

    def use(self, func):
        ''' Append a middleware to the algorithm '''
        # NOTE A middleware Object ?
        # self.use() is usually called from initialize(), so no logger yet
        print 'registering middleware {}'.format(func.__name__)
        self.middlewares.append({
            'call': func,
            'name': func.__name__,
            'args': func.func_code.co_varnames
        })

    # NOTE I'm not superfan of initialize + warm
    def warm(self, data):
        ''' Called at the first handle_data frame '''
        pass

    @abc.abstractmethod
    def event(self, data):
        ''' User should overwrite this method '''
        pass

    def handle_data(self, data):
        ''' Method called for each event by zipline. In intuition this is the
        place to factorize algorithms and then call event() '''
        signals = {}
        self.orderbook = {}

        # Everytime but the first tick
        if self.initialized and self.manager:
            # Keep the portfolio aware of the situation
            self.manager.update(
                self.portfolio,
                self.get_datetime(),
                self.perf_tracker.cumulative_risk_metrics.to_dict())
        else:
            # Perf_tracker needs at least a turn to have an index
            self.sids = data.keys()
            self.warm(data)
            self.initialized = True
            return

        try:
            signals = self.event(data)
        except Exception, error:
            # NOTE Temporary debug. Will probably notify the error and go on
            # with signals={}
            raise intuition.errors.AlgorithmEventFailed(
                reason=error, date=self.get_datetime(), data=data)

        # One can process orders within the alogrithm and don't return anything
        if signals and self.manager:
            if signals.get('buy') or signals.get('sell'):
                self.orderbook = self.manager.trade_signals_handler(signals)
                if self.auto and self._is_interactive():
                    self.process_orders(self.orderbook)

        # Some middlewares send stuff over the wires. This little security
        # prevent us from performing a DDOS
        if self._is_interactive():
            self._call_middlewares()

    def process_orders(self, orderbook):
        ''' Default and costant orders processor. Overwrite it for more
        sophisticated strategies '''
        for stock, alloc in orderbook.iteritems():
            self.logger.info('{}: Ordered {} {} stocks'.format(
                self.get_datetime(), stock, alloc))
            if isinstance(alloc, int):
                self.order(stock, alloc)
            elif isinstance(alloc, float) and \
                    alloc >= -1 and alloc <= 1:
                self.order_percent(stock, alloc)
            else:
                self.logger.warning(
                    '{}: invalid order for {}: {})'
                    .format(self.datetime, stock, alloc))

    @property
    def elapsed_time(self):
        return self.get_datetime() - self.portfolio.start_date

    def _call_one_middleware(self, middleware):
        ''' Evaluate arguments and execute the middleware function '''
        args = {}
        for arg in middleware['args']:
            if hasattr(self, arg):
                # same as eval() but safer for arbitrary code execution
                args[arg] = reduce(getattr, arg.split('.'), self)
        self.logger.debug('calling middleware event {}'
                          .format(middleware['name']))
        middleware['call'](**args)

    def _call_middlewares(self):
        ''' Execute the middleware stack '''
        for middleware in self.middlewares:
            # TODO Call upon datetime conditions
            self._call_one_middleware(middleware)
